#!/bin/bash

ACTION=$1
DEV_TYPE=$2

checkloaded() {
    case $DEV_TYPE in
        ums7|ums8)
            [ `lsmod | grep -w $DEV_TYPE | wc -l` -ne 0 ] && return 1 || return 0
            ;;
        *)
            [ `lsmod | grep -w spc | wc -l` -ne 0 ] && return 1 || return 0
            ;;
    esac
}

remove() {
    case $DEV_TYPE in
        ums7)
            args="/dev/ums7-*"
            ;;
        ums8)
            args="/dev/ums8-*"
            ;;
        *)
            args="/dev/ums7-* /dev/ums8-*"
            ;;
    esac
    rm -f $args >& /dev/null
}

unload() {
    # Removing kernel modules and device files
    if [[ -z $DEV_TYPE ]]; then
        rmmod `lsmod | awk '($1=="spc"){print $4}' | tr ',' ' '` spc || exit 1
    else
        rmmod $DEV_TYPE || exit 1
        lsmod | grep spc | grep ' 0$' >& /dev/null
        if [[ $? -eq 0 ]]; then
            echo "Removing unused spc.ko from memory"
            rmmod spc || exit 1
        fi
    fi
    remove
}

load() {
    checkloaded
    if [ $? == 1 ]
    then
        echo "Already loaded. Use '$0 reload $DEV_TYPE' to reload modules"
        exit 1
    fi

    # Inserting kernel modules
    insmod ./spc.ko >& /dev/null

    if [[ -z $DEV_TYPE ]]; then
        list=`ls *ko | grep -vw 'spc.ko'`
    else
        list=$DEV_TYPE.ko
    fi

    for i in $list; do
        insmod ./$i || echo "Failed to load $i"
    done

    # Creating device files
    MAJOR=`cat /proc/devices | grep spc | awk '{print $1}'`
    [ -z $MAJOR ] && echo 'No spc device found' && exit 1

    UMS7_NR=`lspci -v | grep -w -e ums7 | wc -l`
    UMS8_NR=`lspci -v | grep -w -e ums8 | wc -l`
    case $DEV_TYPE in
        ums7)
            NR=$UMS7_NR
            ;;
        ums8)
            NR=$UMS8_NR
            ;;
        *)
            NR=$(($UMS7_NR + $UMS8_NR))
            ;;
    esac
    [ $NR == 0 ] && echo "No suitable devices found." && exit 1

    remove
    if [ -z $DEV_TYPE ] || [ $DEV_TYPE == "ums7" ]; then
        delta=`ls /dev/ums8* 2>/dev/null | wc -l`
        echo "Got $UMS7_NR devices of 'ums7' type"
        NR=$(( $UMS7_NR - 1 ))
        for i in `seq 0 $NR` ; do
            mknod /dev/ums7-$i c $MAJOR $(( $i + $delta ))
        done
    fi
    if [ -z $DEV_TYPE ] || [ $DEV_TYPE == "ums8" ]; then
        delta=`ls /dev/ums7-* 2>/dev/null | wc -l`
        echo "Got $UMS8_NR devices of 'ums8' type"
        NR=$(( $UMS8_NR - 1 ))
        for i in `seq 0 $NR` ; do
             mknod /dev/ums8-$i c $MAJOR $(( $i + $delta ))
        done
    fi
}

if [[ $UID -ne 0 ]]; then 
    echo "Must be root"
    exit 1
fi

if [[ ! -z $DEV_TYPE ]]; then
    case $DEV_TYPE in
        ums7|ums8)
            echo "Current device type: $DEV_TYPE"
            ;;
        *)
            echo "Unknown device type: $DEV_TYPE"
            echo "Run $0 for more information"
            exit 1
            ;;
   esac
fi

case $1 in
    load|unload)
        $1
        ;;
    reload)
        unload; load
        ;;
    *)
        echo -e "Usage:\n\t$0 load|unload|reload [device_type]\n"
        echo -e "device types:\n\tums7, ums8 or leave it empty for both\n"
        exit 1
        ;;
esac

exit 0
